@@ -678,6 +678,11 @@ static readstat_error_t sav_read_dictionary_termination_record(sav_ctx_t *ctx) {
 }
 
 static readstat_error_t sav_process_row(unsigned char *buffer, size_t buffer_len, sav_ctx_t *ctx) {
+    if (ctx->row_offset) {
+        ctx->row_offset--;
+        return READSTAT_OK;
+    }
+
     readstat_error_t retval = READSTAT_OK;
     double fp_value;
     int offset = 0;
@@ -803,6 +808,14 @@ static readstat_error_t sav_read_uncompressed_data(sav_ctx_t *ctx,
 
     buffer = readstat_malloc(buffer_len);
 
+    if (ctx->row_offset) {
+        if (io->seek(buffer_len * ctx->row_offset, READSTAT_SEEK_CUR, io->io_ctx) == -1) {
+            retval = READSTAT_ERROR_SEEK;
+            goto done;
+        }
+        ctx->row_offset = 0;
+    }
+
     while (ctx->row_limit == -1 || ctx->current_row < ctx->row_limit) {
         retval = sav_update_progress(ctx);
         if (retval != READSTAT_OK)
@@ -876,7 +889,9 @@ static readstat_error_t sav_read_compressed_data(sav_ctx_t *ctx,
                 uncompressed_offset = 0;
             }
 
-            if (state.status == SAV_ROW_STREAM_FINISHED_ALL || ctx->current_row == ctx->row_limit)
+            if (state.status == SAV_ROW_STREAM_FINISHED_ALL)
+                goto done;
+            if (ctx->row_limit > 0 && ctx->current_row == ctx->row_limit)
                 goto done;
         }
     }
@@ -1569,10 +1584,19 @@ readstat_error_t readstat_parse_sav(readstat_parser_t *parser, const char *path,
     ctx->output_encoding = parser->output_encoding;
     ctx->user_ctx = user_ctx;
     ctx->file_size = file_size;
-    if (parser->row_limit > 0 && (parser->row_limit < ctx->record_count || ctx->record_count == -1)) {
+    if (parser->row_offset > 0)
+        ctx->row_offset = parser->row_offset;
+    if (ctx->record_count != -1) {
+        int record_count_after_skipping = ctx->record_count - ctx->row_offset;
+        if (record_count_after_skipping < 0) {
+            record_count_after_skipping = 0;
+            ctx->row_offset = ctx->record_count;
+        }
+        ctx->row_limit = record_count_after_skipping;
+        if (parser->row_limit > 0 && parser->row_limit < record_count_after_skipping) 
+            ctx->row_limit = parser->row_limit;
+    } else if (parser->row_limit > 0) {
         ctx->row_limit = parser->row_limit;
-    } else {
-        ctx->row_limit = ctx->record_count;
     }
     
     if ((retval = sav_parse_timestamp(ctx, &header)) != READSTAT_OK)
